;==========================================================================
;= NUMBER INPUT ROUTINE  By James Vernon                                  =
;=                                                                        =
;=   This routine was originally used in MathPack by James vernon and has =
;= been in existance for some time. This file was uploaded to ticalc.org  =
;= via me(Morgan Davies) in effort to produce more math programs for z80  =
;= calculators. Since I am not the author, give credit when you use this. =
;= James definately deserves it.                                          =
;==========================================================================


;========================================
;=Size: 274 bytes                       =
;=                                      =
;=Input:                                =
;=    a  = currsor position             =
;= (e.x.)   ld a,2                      =
;=    de = ram location to sotre value  =
;= (e.x.)   ld de,op1                   =
;========================================


Input:
    push    de
    ld      de,getNumberBuffer
    ld      (curcol),a
    ld      b,15                            ; B = 15 chars. max
getNumberLoop:
    push    bc                              ; Save counter
    push    de                              ; Save string pointer
    ld      a,$E0
    bcall(_putmap)                          ; Show cursor
    ei
wKeyLoop:
    halt
    bcall(_getcsc)
    or      a
    jr      z,wKeyLoop
    cp      56                              ; [DEL]
    jr      z,numberBackspace
    cp      9                               ; [ENTER]
    jr      z,endNumber
    dec     a
    ld      e,a
    ld      d,0                             ; DE = Offset in CharTable
    ld      hl,getNumberTable
    add     hl,de
    ld      a,(hl)                          ; A = Character entered
    cp      '+'                             ; If it's a "+", it's an invalid character
    jr      z,invalidNumberChar
    push    af
    bcall(_putc)
    pop     af
    pop     de                              ; Restore string pointer
    ld      (de),a                          ; Save character
    inc     de                              ; DE => Where to store next character
    pop     bc
    djnz    getNumberLoop
    jr      endNumberAfterPop
invalidNumberChar:
    pop     de
    pop     bc
    jr      getNumberLoop
numberBackspace:
    pop     de                              ; Restore string pointer
    pop     bc
    ld      a,b
    cp      15
    jr      z,getNumberLoop
    inc     b
    dec     de
    push    bc
    push    de
    ld      a,' '
    bcall(_putmap)
    pop     de
    pop     bc
    ld      hl,curcol
    dec     (hl)
    jr      getNumberLoop
endNumber:
    pop     de                              ; Restore string pointer
    pop     bc
endNumberAfterPop:
    xor     a                               ; NULL terminator
    ld      (de),a                          ; Save it
    ld      a,' '
    bcall(_putmap)                          ; Erase cursor character
; Clear where to store floating point value
    pop     de
    push    de
    ld      h,d
    ld      l,e
    inc     de
    ld      bc,11
    ld      (hl),0
    ldir
; Check it's a valid number
preCheckValidNumber:
    ld      hl,getNumberBuffer
    ld      bc,$FEFE
    ld      d,0
checkValidNumber:
    ld      a,(hl)
    cp      '.'
    jr      nz,cvnNotDecimal
    inc     b
    jr      z,invalidNumber
    ld      e,a
    ld      a,d
    add     a,$7F
    ld      (_gnExponent),a
    ld      a,e
cvnNotDecimal:
    cp      $1A
    jr      nz,cvnNotNeg
    inc     c
    jr      z,invalidNumber
    ld      e,a
    ld      a,d
    or      a
    jr      nz,invalidNumber
    ld      a,e
    dec     d
cvnNotNeg:
    or      a
    jr      z,endNumberString
    inc     hl
    inc     d
    jr      checkValidNumber
endNumberString:
; Format number and write to Op var
    inc     b
    jr      z,gnGetSign
    ld      (hl),'.'
    inc     hl
    ld      (hl),'0'
    inc     hl
    ld      (hl),0
    jr      preCheckValidNumber
gnGetSign:
    pop     de                              ; DE => Sign byte of Op var
    inc     c
    ld      a,$80                           ; Negative
    jr      z,getNumberSetSign
    xor     a
getNumberSetSign:
    ld      (de),a
    inc     de                              ; DE => Exponent
    ld      a,$80                           ; Exponent value
_gnExponent             = $-1
    ld      (de),a                          ; Save exponent
    inc     de                              ; DE => First byte of mantissa
    push    de
    ld      l,e
    ld      h,d
    inc     de
    ld      bc,6
    ld      (hl),0
    ldir
    pop     de
    ld      hl,getNumberBuffer
    ex      de,hl
getNumberFormat:
    call    getNumberCharacter
    or      a
    ret     z
    sub     '0'
    add     a,a
    add     a,a
    add     a,a
    add     a,a
    or      (hl)
    ld      (hl),a
    call    getNumberCharacter
    or      a
    ret     z
    sub     '0'
    or      (hl)
    ld      (hl),a
    inc     hl                          ; HL => Next byte of mantissa
    jr      getNumberFormat
invalidNumber:
    bcall(_newline)
    pop     de
    jp      Input
getNumberCharacter:
    ld      a,(de)
    inc     de
    cp      $1A
    jr      z,getNumberCharacter
    cp      '.'
    jr      z,getNumberCharacter
    ret
getNumberTable:
.db "++++++++++"
.db "++++++",$1A,"369"
.db "++++.258++"
.db "++0147++++"
.db "++++++++++"
.db "+++++"

.end